/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "AskPassQt.h"
#include "AskPassQt.moc"

#include "AnimatedLineEditor.h"
#include "CompositeEngine.h"
#include "RoundedRegion.h"
#include "ShadowLabel.h"
#include "SvgEngine.h"
#include "WinUtil.h"
#include "X11Util.h"

#include <QFile>
#include <QApplication>
#include <QDesktopWidget>
#include <QLabel>
#include <QLayout>
#include <QToolButton>
#include <QPainter>

#include <cstdio>
#include <iostream>

//______________________________________________________
AskPassQt::AskPassQt( QWidget* parent ) :
    TRANSPARENCY::TransparentWidget( parent ),
    status_( 1 ),
    button_( Qt::NoButton )
{
    setWindowTitle( "Ssh Password" );
    setWindowFlags( Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint );

    // opaque background
    setAttribute( Qt::WA_OpaquePaintEvent, true );
    setAttribute( Qt::WA_StyledBackground, false );
    setAttribute( Qt::WA_TranslucentBackground, true );

    // main layout
    QVBoxLayout* mainLayout = new QVBoxLayout();
    setLayout( mainLayout );
    layout()->setMargin(0);
    layout()->setSpacing(5);

    QHBoxLayout *hLayout( new QHBoxLayout() );
    hLayout->setSpacing(10);
    hLayout->setMargin(0);
    layout()->addItem( hLayout );

    // pixmap
    QPixmap pixmap( ":/icons/32x32/dialog-password.png" );
    if( !pixmap.isNull() )
    {
        QLabel* label = new QLabel( this );
        label->setMargin(10);
        label->setPixmap( pixmap );
        hLayout->addWidget( label, 0, Qt::AlignHCenter );
    }

    QVBoxLayout *vLayout( new QVBoxLayout() );
    vLayout->setSpacing(2);
    vLayout->setMargin(0);
    hLayout->addItem( vLayout );

    // label
    {

        TRANSPARENCY::ShadowLabel* label = new TRANSPARENCY::ShadowLabel( tr( "Enter your SSH pass phrase:" ), this );
        vLayout->addWidget( label );
        label->setShadowColor( Qt::white );
        label->setShadowOffset( 1 );
    }

    QPalette palette( Qt::black );

    vLayout->addWidget( editor_ = new AnimatedLineEditor( this ) );
    editor_->setEchoMode( QLineEdit::Password );
    connect( editor_, SIGNAL(returnPressed()), SLOT(accept()) );

    mainLayout->addSpacing( 5 );

    // buttons
    hLayout = new QHBoxLayout();
    hLayout->setSpacing(10);
    hLayout->setMargin(0);
    layout()->addItem( hLayout );

    // stretch
    hLayout->addStretch( 1 );
    QToolButton* button;

    // ok button
    hLayout->addWidget( button = new QToolButton( this ) );
    button->setIcon( QIcon( ":/icons/32x32/accept.png") );
    button->setText( tr( "OK" ) );
    button->setAutoRaise( true );
    button->setToolButtonStyle( Qt::ToolButtonTextBesideIcon );
    button->setIconSize( QSize( 22, 22 ) );
    connect( button, SIGNAL(clicked()), SLOT(accept()) );

    hLayout->addWidget( button = new QToolButton( this ) );
    button->setIcon( QIcon( ":/icons/32x32/cancel.png") );
    button->setShortcut( Qt::Key_Escape );
    button->setText( tr( "Cancel" ) );
    button->setAutoRaise( true );
    button->setToolButtonStyle( Qt::ToolButtonTextBesideIcon );
    button->setIconSize( QSize( 22, 22 ) );
    connect( button, SIGNAL(clicked()), SLOT(cancel()) );

    // set focus
    editor_->setFocus();

    // initialize svg engine
    SVG::SvgEngine::get().reload();

    // margins
    BASE::Margins margins( SVG::SvgEngine::get().margins() );
    setContentsMargins( margins.left(), margins.top(), margins.right(), margins.bottom() );
    _setMargins( margins );

    // outer padding
    _setOuterPadding( SVG::SvgEngine::get().outerPadding() );

    // resize
    QSize size( 400, 100 );
    resize( size );

    // move widget at desktop center
    QPoint position(
        ( ( QApplication::desktop()->width() - size.width() ) / 2 ),
        ( ( QApplication::desktop()->height() - size.height() ) / 2 ) );
    move( position );

}

//______________________________________________________
void AskPassQt::paintEvent( QPaintEvent* event )
{

    #if defined(Q_OS_WIN)

    QPixmap pixmap( size() );
    pixmap.fill( Qt::transparent );
    _paint( pixmap, event->rect() );
    WinUtil( this ).update( pixmap, 1 );

    #else

    _paint( *this, event->rect() );

    #endif

}

//______________________________________________________
void AskPassQt::_paint( QPaintDevice& device, QRect clip ) const
{

    // draw background
    QPainter painter( &device );

    #if !defined(Q_OS_WIN)
    painter.setClipRect( clip );
    #endif

    // draw background
    if( TRANSPARENCY::CompositeEngine::get().isAvailable() )
    {

        painter.setRenderHints(QPainter::SmoothPixmapTransform);
        painter.setCompositionMode(QPainter::CompositionMode_Source );
        painter.fillRect( rect(), Qt::transparent );

    }

    // draw svg
    painter.drawPixmap( QPoint( 0, 0 ), SVG::SvgEngine::get().get( size() ) );

    // horizontal separator
    int y( editor_->mapToParent( editor_->rect().bottomLeft() ).y() + 8 );

    const BASE::Margins margins( SVG::SvgEngine::get().margins() );

    painter.setRenderHint(QPainter::Antialiasing, false );
    painter.setCompositionMode( QPainter::CompositionMode_SourceOver );
    QLinearGradient gradient( QPointF( margins.left(), 0 ), QPointF( width() - margins.width(), 0 ) );
    gradient.setColorAt(0, Qt::transparent );
    gradient.setColorAt(0.3, "#cccccc" );
    gradient.setColorAt(0.7, "#cccccc" );
    gradient.setColorAt(1, Qt::transparent );
    painter.setBrush( gradient );
    painter.setPen( Qt::NoPen );
    painter.drawRect( QRectF( QPointF( margins.left(), y ), QSizeF( width()-margins.width(), 1 ) ) );

    gradient.setColorAt(0, Qt::transparent );
    gradient.setColorAt(0.3, "#555555" );
    gradient.setColorAt(0.7, "#555555" );
    gradient.setColorAt(1, Qt::transparent );
    painter.setBrush( gradient );
    painter.setPen( Qt::NoPen );
    painter.drawRect( QRectF( QPointF( margins.left(), y-1 ), QSizeF( width()-margins.width(),1 ) ) );

    painter.end();

    // loop over children and render them
    #if defined(Q_OS_WIN)
    QList<QWidget*> widgets( findChildren<QWidget*>() );
    for( QList<QWidget*>::iterator iter = widgets.begin(); iter != widgets.end(); ++iter )
    { (*iter)->render( &device, (*iter)->geometry().topLeft(), (*iter)->rect().intersected( rect().translated( -(*iter)->geometry().topLeft() ) ), 0 ); }
    #endif

}


//______________________________________________________
void AskPassQt::resizeEvent( QResizeEvent* event )
{
    setMask( RoundedRegion( rect(), RoundedRegion::All ) );

    // announce blur region
    BASE::Margins outerPadding( SVG::SvgEngine::get().outerPadding() );
    if( outerPadding.isNull() )
    {
        outerPadding = SVG::SvgEngine::get().margins();
        outerPadding -= BASE::Margins( 4 );
    }

    _updateBlurRegion( outerPadding.adjustedRect( rect() ) );

    return QWidget::resizeEvent( event );
}


//___________________________________________________
void AskPassQt::mousePressEvent( QMouseEvent *event )
{

    QWidget::mousePressEvent( event );

    _setButton( event->button() );
    if( _button() == Qt::LeftButton )
    {

        clickPosition_ = event->pos() + QPoint(geometry().topLeft() - frameGeometry().topLeft());

        // try use X11 Window Manager to handle event
        if( X11Util::get().moveResizeWidget( this, event->globalPos(), X11Util::_NET_WM_MOVERESIZE_MOVE ) ) return;
        else setCursor( Qt::SizeAllCursor );

    }

}

//___________________________________________________
void AskPassQt::mouseReleaseEvent( QMouseEvent* )
{
    _setButton( Qt::NoButton );
    unsetCursor();
}

//___________________________________________________
void AskPassQt::mouseMoveEvent( QMouseEvent *event )
{

    if( _button() == Qt::LeftButton )
    {

        QPoint point( event->globalPos() - clickPosition_ );
        move( point );

    }
}

//______________________________________________________
void AskPassQt::accept( void )
{
    puts( editor_->text().toUtf8() );
    status_ = 0;
    close();
}

//______________________________________________________
void AskPassQt::cancel( void )
{ exit( status_ ); }
